@@ -40,6 +40,7 @@ from debputy.exceptions import (
     PluginInitializationError,
     UnhandledOrUnexpectedErrorFromPluginError,
     SymlinkLoopError,
+    DebputyRuntimeErrorWithPreamble,
 )
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.manifest_parser.parser_doc import render_rule
@@ -1106,9 +1107,9 @@ def main() -> None:
         ROOT_COMMAND(cmd_arg)
     except PluginInitializationError as e:
         _error_w_stack_trace(
-            "Failed to load a plugin - full stack strace:",
+            "Failed to load a plugin - relevant stack trace (use --debug/DEBPUTY_DEBUG=1 to see the full one):",
             e.message,
-            e,
+            e.__cause__ if e.__cause__ else e,
             parsed_args.debug_mode,
             follow_warning=[
                 "Please consider filing a bug against the plugin in question"
@@ -1158,6 +1159,8 @@ def main() -> None:
                 "Re-raising original exception to show the full stack trace due to debug mode being active"
             )
             raise e
+        if isinstance(e, DebputyRuntimeErrorWithPreamble):
+            e.render_preamble()
         _error(e.message)
     except AssertionError as e:
         _error_w_stack_trace(
