@@ -2,7 +2,6 @@ import dataclasses
 import functools
 import os
 import textwrap
-from collections import defaultdict
 from contextlib import suppress
 from dataclasses import dataclass, field
 from typing import (
@@ -33,7 +32,11 @@ from ._deb_options_profiles import DebBuildOptionsAndProfiles
 from ._manifest_constants import *
 from .architecture_support import DpkgArchitectureBuildProcessValuesTable
 from .builtin_manifest_rules import builtin_mode_normalization_rules
-from .exceptions import DebputySubstitutionError
+from .exceptions import (
+    DebputySubstitutionError,
+    DebputyRuntimeError,
+    DebputyRuntimeErrorWithPreamble,
+)
 from .filesystem_scan import FSPath, FSRootDir, FSROOverlay
 from .installations import (
     InstallRule,
@@ -92,6 +95,27 @@ from .yaml import MANIFEST_YAML
 from .yaml.compat import CommentedMap, CommentedSeq
 
 
+class PathNotCoveredByInstallRulesError(DebputyRuntimeErrorWithPreamble):
+
+    @property
+    def unmatched_paths(self) -> Sequence[VirtualPath]:
+        return self.args[1]
+
+    @property
+    def search_dir(self) -> VirtualPath:
+        return self.args[2]
+
+    def render_preamble(self) -> None:
+        _warn(
+            f"The following paths were present in {self.search_dir.fs_path}, but not installed (nor explicitly discarded)."
+        )
+        _warn("")
+        for entry in self.unmatched_paths:
+            desc = _describe_missing_path(entry)
+            _warn(f" * {desc}")
+        _warn("")
+
+
 @dataclass(slots=True)
 class DbgsymInfo:
     binary_package: BinaryPackage
@@ -1005,31 +1029,24 @@ def _detect_missing_installations(
     path_matcher: SourcePathMatcher,
     search_dir: VirtualPath,
 ) -> None:
-    if not os.path.isdir(search_dir.fs_path):
+    if not search_dir.is_dir:
         return
     missing = list(path_matcher.detect_missing(search_dir))
     if not missing:
         return
 
-    _warn(
-        f"The following paths were present in {search_dir.fs_path}, but not installed (nor explicitly discarded)."
-    )
-    _warn("")
-    for entry in missing:
-        desc = _describe_missing_path(entry)
-        _warn(f" * {desc}")
-    _warn("")
-
     excl = textwrap.dedent(
         """\
         - discard: "*"
     """
     )
 
-    _error(
+    raise PathNotCoveredByInstallRulesError(
         "Please review the list and add either install rules or exclusions to `installations` in"
         " debian/debputy.manifest.  If you do not need any of these paths, add the following to the"
-        f" end of your 'installations`:\n\n{excl}\n"
+        f" end of your 'installations`:\n\n{excl}\n",
+        missing,
+        search_dir,
     )
 
 
