@@ -6,6 +6,7 @@ import os
 import time
 from collections import defaultdict, Counter
 from enum import IntEnum
+from functools import lru_cache
 from typing import (
     List,
     Optional,
@@ -68,6 +69,7 @@ FormatterImpl = Callable[["LintState"], Optional[Sequence[types.TextEdit]]]
 DIAG_SOURCE_WITHOUT_SECTIONS: FrozenSet[str] = frozenset(
     {
         "debputy",
+        "dpkg",
     }
 )
 
@@ -94,6 +96,7 @@ def te_range_to_lsp(te_range: "TERange") -> types.Range:
     )
 
 
+@lru_cache
 def _check_diagnostic_source(source: str) -> None:
     if source in DIAG_SOURCE_WITHOUT_SECTIONS:
         return
@@ -101,7 +104,7 @@ def _check_diagnostic_source(source: str) -> None:
     s = parts[0]
     if s not in DIAG_SOURCE_WITH_SECTIONS:
         raise ValueError(
-            f"Unknown diagnostic source: {source}. If you are adding a new source, update lint_util.py"
+            f'Unknown diagnostic source: "{source}". If you are adding a new source, update lint_util.py'
         )
     if len(parts) != 2:
         raise ValueError(
@@ -248,8 +251,8 @@ class LintState:
         ignored_words = set()
         source_package = self.source_package
         binary_packages = self.binary_packages
-        if source_package:
-            ignored_words.add(source_package.name)
+        if source_package and source_package.fields.get("Source") is not None:
+            ignored_words.add(source_package.fields.get("Source"))
         if binary_packages:
             ignored_words.update(binary_packages)
         return checker.context_ignored_words(ignored_words)
@@ -270,6 +273,7 @@ class LintState:
         tags: Optional[List[types.DiagnosticTag]] = None,
         related_information: Optional[List[types.DiagnosticRelatedInformation]] = None,
         diagnostic_applies_to_another_file: Optional[str] = None,
+        enable_non_interactive_auto_fix: bool = True,
     ) -> None:
         """Emit a diagnostic for an issue detected in the current file.
 
@@ -290,6 +294,9 @@ class LintState:
           Use the quickfixes provided in `debputy.lsp.quickfixes` such as `propose_correct_text_quick_fix`.
         :param tags: TODO: Not yet specified (currently uses LSP format).
         :param related_information: TODO: Not yet specified (currently uses LSP format).
+        :param enable_non_interactive_auto_fix: Allow non-interactive auto-fixing (such as via
+          `debputy lint --auto-fix`) of this issue. Set to `False` if the check is likely to have false
+          positives.
         :param diagnostic_applies_to_another_file: Special-case parameter for flagging invalid file names.
           Leave this one at `None`, unless you know you need it.
 
@@ -298,7 +305,9 @@ class LintState:
         """
         _check_diagnostic_source(authority_reference)
         lsp_severity = LINT_SEVERITY2LSP_SEVERITY[severity]
-        diag_data: DiagnosticData = {}
+        diag_data: DiagnosticData = {
+            "enable_non_interactive_auto_fix": enable_non_interactive_auto_fix,
+        }
 
         if severity not in NATIVELY_LSP_SUPPORTED_SEVERITIES:
             diag_data["lint_severity"] = severity
@@ -422,8 +431,9 @@ class LintStateImpl(LintState):
 
 class LintDiagnosticResultState(IntEnum):
     REPORTED = 1
-    FIXABLE = 2
-    FIXED = 3
+    MANUAL_FIXABLE = 2
+    AUTO_FIXABLE = 3
+    FIXED = 4
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
@@ -649,17 +659,22 @@ class TermLintReport(LintReport):
             end_line = diagnostic.range.end.line
             end_position = diagnostic.range.end.character
 
-        has_fixit = ""
+        authority = diagnostic.source
+        assert authority is not None
+        diag_tags = f" [{authority}]"
         lines = lint_state.lines
         line_no_width = len(str(len(lines)))
 
-        if diagnostic_result.result_state == LintDiagnosticResultState.FIXABLE:
-            has_fixit = " [Correctable via --auto-fix]"
+        if diagnostic_result.result_state == LintDiagnosticResultState.AUTO_FIXABLE:
+            diag_tags += "[Correctable via --auto-fix]"
+        elif diagnostic_result.result_state == LintDiagnosticResultState.MANUAL_FIXABLE:
+            diag_tags += "[LSP interactive quickfix]"
 
         code = f"[{diagnostic.code}]: " if diagnostic.code else ""
         msg = f"{code}{diagnostic.message}"
+
         print(
-            f"{tag}: File: {filename}:{start_line+1}:{start_position}:{end_line+1}:{end_position}: {msg}{has_fixit}",
+            f"{tag}: File: {filename}:{start_line+1}:{start_position}:{end_line+1}:{end_position}: {msg}{diag_tags}",
         )
         if diagnostic_result.missing_severity:
             _warn(
