@@ -172,15 +172,25 @@ def _check_header_line(
         and dctrl_source_pkg is not None
         and dctrl_source_pkg.fields.get("Source") != source_name
     ):
-        expected_name = dctrl_source_pkg.fields.get("Source") or "(missing)"
+        expected_name = dctrl_source_pkg.fields.get("Source")
         start_pos, end_pos = m.span(1)
         name_range = _single_line_subrange(line_no, start_pos, end_pos)
+        if expected_name is None:
+            msg = (
+                "The first entry must use the same source name as debian/control."
+                ' The d/control file is missing the "Source" field in its first stanza'
+            )
+        else:
+            msg = (
+                "The first entry must use the same source name as debian/control."
+                f' Changelog uses: "{source_name}" while d/control uses: "{expected_name}"'
+            )
+
         lint_state.emit_diagnostic(
             name_range,
-            f"The first entry must use the same source name as debian/control."
-            f' Changelog uses: "{source_name}" while d/control uses: "{expected_name}"',
+            msg,
             "error",
-            "debputy",
+            "dpkg",  # man:deb-src-control(5) / #1089794
         )
     if not PKGVERSION_REGEX.fullmatch(source_version):
         vm = PKGVERSION_REGEX.search(source_version)
@@ -209,7 +219,7 @@ def _check_header_line(
                     r,
                     "This part cannot be parsed as a valid Debian version.",
                     "error",
-                    "debputy",
+                    "Policy 5.6.12",
                 )
         else:
             name_range = _single_line_subrange(line_no, start_pos, end_pos)
@@ -217,7 +227,7 @@ def _check_header_line(
                 name_range,
                 f'Cannot parse "{source_version}" as a Debian version.',
                 "error",
-                "debputy",
+                "Policy 5.6.12",
             )
     elif "dsfg" in source_version:
         typo_index = source_version.index("dsfg")
