@@ -21,7 +21,7 @@ from debputy.lsp.lsp_features import (
     LanguageDispatchRule,
 )
 from debputy.lsp.lsp_generic_yaml import (
-    word_range_at_position,
+    error_range_at_position,
     YAML_COMPLETION_HINT_KEY,
     insert_complete_marker_snippet,
     yaml_key_range,
@@ -63,6 +63,7 @@ from debputy.plugin.api.impl_types import (
     ListWrappedDeclarativeInputParser,
     InPackageContextParser,
     DeclarativeValuelessKeywordInputParser,
+    PluginProvidedParser,
 )
 from debputy.plugin.api.parser_tables import OPARSER_MANIFEST_ROOT
 from debputy.plugin.api.spec import DebputyIntegrationMode
@@ -105,11 +106,11 @@ def _lint_debian_debputy_manifest(lint_state: LintState) -> None:
     except MarkedYAMLError as e:
         if e.context_mark:
             line = e.context_mark.line
-            column = e.context_mark.column + 1
+            column = e.context_mark.column
         else:
             line = e.problem_mark.line
-            column = e.problem_mark.column + 1
-        error_range = word_range_at_position(
+            column = e.problem_mark.column
+        error_range = error_range_at_position(
             lines,
             line,
             column,
@@ -435,7 +436,7 @@ def _lint_content(
         known_packages = lint_state.binary_packages
         lc = content.lc
         for k, v in content.items():
-            if "{{" not in k and known_packages is not None and k not in known_packages:
+            if k is None or ("{{" not in k and known_packages is not None and k not in known_packages):
                 line, col = lc.key(k)
                 yaml_flag_unknown_key(
                     lint_state,
@@ -462,6 +463,30 @@ def _lint_content(
         )
 
 
+def keywords_with_parser(
+    parser: Union[DeclarativeMappingInputParser, DispatchingParserBase],
+) -> Tuple[str, PluginProvidedParser]:
+    for keyword in parser.registered_keywords():
+        pp_subparser = parser.parser_for(keyword)
+        yield keyword, pp_subparser
+
+
+def completion_item(
+    quoted_keyword: str,
+    pp_subparser: PluginProvidedParser,
+) -> CompletionItem:
+    inline_reference_documentation = pp_subparser.parser.inline_reference_documentation
+    synopsis = (
+        inline_reference_documentation.synopsis
+        if inline_reference_documentation
+        else None
+    )
+    return CompletionItem(
+        quoted_keyword,
+        detail=synopsis,
+    )
+
+
 @lsp_completer(_DISPATCH_RULE)
 def debputy_manifest_completer(
     ls: "DebputyLanguageServer",
@@ -542,25 +567,28 @@ def debputy_manifest_completer(
         if isinstance(parser, DispatchingParserBase):
             if matched_key:
                 items = [
-                    CompletionItem(
-                        maybe_quote_yaml_value(k)
-                        if has_colon
-                        else f"{maybe_quote_yaml_value(k)}:"
+                    completion_item(
+                        (
+                            maybe_quote_yaml_value(k)
+                            if has_colon
+                            else f"{maybe_quote_yaml_value(k)}:"
+                        ),
+                        pp_subparser,
                     )
-                    for k in parser.registered_keywords()
+                    for k, pp_subparser in keywords_with_parser(parser)
                     if k not in parent
                     and not isinstance(
-                        parser.parser_for(k).parser,
+                        pp_subparser.parser,
                         DeclarativeValuelessKeywordInputParser,
                     )
                 ]
             else:
                 items = [
-                    CompletionItem(maybe_quote_yaml_value(k))
-                    for k in parser.registered_keywords()
+                    completion_item(maybe_quote_yaml_value(k), pp_subparser)
+                    for k, pp_subparser in keywords_with_parser(parser)
                     if k not in parent
                     and isinstance(
-                        parser.parser_for(k).parser,
+                        pp_subparser.parser,
                         DeclarativeValuelessKeywordInputParser,
                     )
                 ]
