@@ -113,6 +113,18 @@ _KEY2FIELD_VALUE_CLASS = {x.key: x for x in FieldValueClass}
 class Documentation(DebputyParsedContent):
     synopsis: NotRequired[str]
     long_description: NotRequired[str]
+    uris: NotRequired[List[str]]
+
+
+class DCtrlSubstvar(DebputyParsedContent):
+    name: str
+    defined_by: str
+    dh_sequence: NotRequired[str]
+    documentation: NotRequired[Documentation]
+
+
+class DctrlSubstvarsReferenceData(DebputyParsedContent):
+    substvars: List[DCtrlSubstvar]
 
 
 class StaticValue(DebputyParsedContent):
@@ -158,3 +170,8 @@ class Deb822ReferenceData(DebputyParsedContent):
 DEB822_REFERENCE_DATA_PARSER = _DEB822_REFERENCE_DATA_PARSER_GENERATOR.generate_parser(
     Deb822ReferenceData
 )
+
+
+DCTRL_SUBSTVARS_REFERENCE_DATA_PARSER = (
+    _DEB822_REFERENCE_DATA_PARSER_GENERATOR.generate_parser(DctrlSubstvarsReferenceData)
+)
