@@ -2,6 +2,7 @@ import contextlib
 import dataclasses
 import functools
 import importlib
+import importlib.resources
 import importlib.util
 import itertools
 import json
@@ -10,7 +11,9 @@ import re
 import subprocess
 import sys
 from abc import ABC
+from importlib.resources.abc import Traversable
 from json import JSONDecodeError
+from pathlib import Path
 from typing import (
     Optional,
     Callable,
@@ -56,6 +59,11 @@ from debputy.manifest_parser.parser_data import ParserContextData
 from debputy.manifest_parser.tagging_types import TypeMapping
 from debputy.manifest_parser.util import AttributePath
 from debputy.manifest_parser.util import resolve_package_type_selectors
+from debputy.plugin.api.doc_parsing import (
+    DEBPUTY_DOC_REFERENCE_DATA_PARSER,
+    parser_type_name,
+    DebputyParsedDoc,
+)
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
 from debputy.plugin.api.impl_types import (
     DebputyPluginMetadata,
@@ -112,7 +120,6 @@ from debputy.plugin.api.spec import (
     packager_provided_file_reference_documentation,
     TypeMappingDocumentation,
     DebputyIntegrationMode,
-    reference_documentation,
     _DEBPUTY_DISPATCH_METADATA_ATTR_NAME,
     BuildSystemManifestRuleMetadata,
     INTEGRATION_MODE_FULL,
@@ -120,7 +127,6 @@ from debputy.plugin.api.spec import (
 )
 from debputy.plugin.api.std_docs import _STD_ATTR_DOCS
 from debputy.plugin.debputy.to_be_api_types import (
-    BuildSystemRule,
     BuildRuleParsedFormat,
     BSPF,
     debputy_build_system,
@@ -142,7 +148,9 @@ from debputy.util import (
     _error,
     print_command,
     _warn,
+    _debug_log,
 )
+from debputy.yaml import MANIFEST_YAML
 
 if TYPE_CHECKING:
     from debputy.highlevel_manifest import HighLevelManifest
@@ -207,6 +215,8 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         "_plugin_detector_ids",
         "_substitution",
         "_unloaders",
+        "_is_doc_cache_resolved",
+        "_doc_cache",
         "_load_started",
     )
 
@@ -221,6 +231,8 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
         self._plugin_detector_ids: Set[str] = set()
         self._substitution = substitution
         self._unloaders: List[Callable[[], None]] = []
+        self._is_doc_cache_resolved: bool = False
+        self._doc_cache: Optional[DebputyParsedDoc] = None
         self._load_started = False
 
     def unload_plugin(self) -> None:
@@ -261,6 +273,66 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                 f"Exception while attempting to load plugin {metadata.plugin_name}"
             ) from e
 
+    def _resolve_docs(self) -> Optional[DebputyParsedDoc]:
+        doc_cache = self._doc_cache
+        if doc_cache is not None:
+            return doc_cache
+
+        plugin_doc_path = self._plugin_metadata.plugin_doc_path
+        if plugin_doc_path is None or self._is_doc_cache_resolved:
+            self._is_doc_cache_resolved = True
+            return None
+        try:
+            with plugin_doc_path.open("r", encoding="utf-8") as fd:
+                raw = MANIFEST_YAML.load(fd)
+        except FileNotFoundError:
+            _debug_log(
+                f"No documentation file found for {self._plugin_name}. Expected it at {plugin_doc_path}"
+            )
+            self._is_doc_cache_resolved = True
+            return None
+        attr_path = AttributePath.root_path(plugin_doc_path)
+        try:
+            ref = DEBPUTY_DOC_REFERENCE_DATA_PARSER.parse_input(raw, attr_path)
+        except ManifestParseException as e:
+            raise ValueError(
+                f"Could not parse documentation in {plugin_doc_path}: {e.message}"
+            ) from e
+        try:
+            res = DebputyParsedDoc.from_ref_data(ref)
+        except ValueError as e:
+            raise ValueError(
+                f"Could not parse documentation in {plugin_doc_path}: {e.args[0]}"
+            ) from e
+
+        self._doc_cache = res
+        self._is_doc_cache_resolved = True
+        return res
+
+    def _pluggable_manifest_docs_for(
+        self,
+        rule_type: Union[TTP, str],
+        rule_name: Union[str, List[str]],
+        *,
+        inline_reference_documentation: Optional[ParserDocumentation] = None,
+    ) -> Optional[ParserDocumentation]:
+        ref_data = self._resolve_docs()
+        if ref_data is not None:
+            primary_rule_name = (
+                rule_name if isinstance(rule_name, str) else rule_name[0]
+            )
+            rule_ref = f"{parser_type_name(rule_type)}::{primary_rule_name}"
+            resolved_docs = ref_data.pluggable_manifest_rules.get(rule_ref)
+            if resolved_docs is not None:
+                if inline_reference_documentation is not None:
+                    raise ValueError(
+                        f"Conflicting docs for {rule_ref}: Was provided one in the API call and one via"
+                        f" {self._plugin_metadata.plugin_doc_path}. Please remove one of the two, so"
+                        f" there is only one doc reference"
+                    )
+                return resolved_docs
+        return inline_reference_documentation
+
     def packager_provided_file(
         self,
         stem: str,
@@ -797,6 +869,13 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
             raise ValueError(
                 f"The rule_type was not a supported type. It must be one of {types}"
             )
+
+        inline_reference_documentation = self._pluggable_manifest_docs_for(
+            rule_type,
+            rule_name,
+            inline_reference_documentation=inline_reference_documentation,
+        )
+
         dispatching_parser = parser_generator.dispatchable_table_parsers[rule_type]
         dispatching_parser.register_keyword(
             rule_name,
@@ -905,6 +984,12 @@ class DebputyPluginInitializerProvider(DebputyPluginInitializer):
                 )
             dispatching_parser = parser_generator.dispatchable_table_parsers[rule_type]
 
+        inline_reference_documentation = self._pluggable_manifest_docs_for(
+            rule_type,
+            rule_name,
+            inline_reference_documentation=inline_reference_documentation,
+        )
+
         if apply_standard_attribute_documentation:
             docs = _STD_ATTR_DOCS
         else:
@@ -1521,6 +1606,18 @@ class BinaryCtrlAccessorProviderCreator:
         return self._triggers.values()
 
 
+def _resolve_bundled_plugin_docs_path(
+    plugin_name: str,
+    loader: Optional[PluginInitializationEntryPoint],
+) -> Optional[Union[Traversable, Path]]:
+    plugin_module = getattr(loader, "__module__")
+    assert plugin_module is not None
+    plugin_package_name = sys.modules[plugin_module].__package__
+    return importlib.resources.files(plugin_package_name).joinpath(
+        f"{plugin_name}_docs.yaml"
+    )
+
+
 def plugin_metadata_for_debputys_own_plugin(
     loader: Optional[PluginInitializationEntryPoint] = None,
 ) -> DebputyPluginMetadata:
@@ -1528,11 +1625,16 @@ def plugin_metadata_for_debputys_own_plugin(
         from debputy.plugin.debputy.debputy_plugin import initialize_debputy_features
 
         loader = initialize_debputy_features
+    plugin_name = "debputy"
     return DebputyPluginMetadata(
         plugin_name="debputy",
         api_compat_version=1,
         plugin_initializer=loader,
         plugin_loader=None,
+        plugin_doc_path_resolver=lambda: _resolve_bundled_plugin_docs_path(
+            plugin_name,
+            loader,
+        ),
         plugin_path="<bundled>",
     )
 
@@ -1997,8 +2099,18 @@ def _open(path: str, fd: Optional[IO[bytes]] = None) -> Iterator[IO[bytes]]:
             yield fd
 
 
+def _resolve_json_plugin_docs_path(
+    plugin_name: str,
+    plugin_path: str,
+) -> Optional[Union[Traversable, Path]]:
+    plugin_dir = os.path.dirname(plugin_path)
+    return Path(os.path.join(plugin_dir, plugin_name + "_docs.yaml"))
+
+
 def parse_json_plugin_desc(
-    path: str, *, fd: Optional[IO[bytes]] = None
+    path: str,
+    *,
+    fd: Optional[IO[bytes]] = None,
 ) -> DebputyPluginMetadata:
     with _open(path, fd=fd) as rfd:
         try:
@@ -2042,6 +2154,9 @@ def parse_json_plugin_desc(
             attribute_path,
         ),
         api_compat_version=api_compat,
+        plugin_doc_path_resolver=lambda: _resolve_json_plugin_docs_path(
+            plugin_name, path
+        ),
         plugin_initializer=None,
         plugin_path=path,
     )
