@@ -47,7 +47,7 @@ class LintWrapper:
         self.lint_maint_preference_table = MaintainerPreferenceTable({}, {})
         self.effective_preference: Optional[EffectiveFormattingPreference] = None
 
-    def __call__(self, lines: List[str]) -> Optional[List["Diagnostic"]]:
+    def __call__(self, lines: List[str]) -> List["Diagnostic"]:
         source_package = None
         binary_packages = None
         dctrl_lines = self.dctrl_lines
@@ -83,11 +83,13 @@ def requires_levenshtein(func: Any) -> Any:
 
 def check_diagnostics(
     diagnostics: Optional[List["Diagnostic"]],
-) -> Optional[List["Diagnostic"]]:
+) -> List["Diagnostic"]:
     if diagnostics:
         for diagnostic in diagnostics:
             assert diagnostic.severity is not None
             assert diagnostic.source is not None
+    elif diagnostics is None:
+        diagnostics = []
     return diagnostics
 
 
